/*
 * Copyright (c) 1998-2012 Caucho Technology -- all rights reserved
 *
 * This file is part of Resin(R) Open Source
 *
 * Each copy or derived work must preserve the copyright notice and this
 * notice unmodified.
 *
 * Resin Open Source is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Resin Open Source is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, or any warranty
 * of NON-INFRINGEMENT.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Resin Open Source; if not, write to the
 *
 *   Free Software Foundation, Inc.
 *   59 Temple Place, Suite 330
 *   Boston, MA 02111-1307  USA
 *
 * @author Emil Ong
 */

package com.caucho.xtpdoc;

import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;
import java.io.IOException;
import java.io.PrintWriter;

/**
 * Creates a fixed javascript legend that hides and shows reference data
 * sections such as schemas, descriptions, etc.
 */
public class ReferenceLegend implements ContentItem {
  public void writeHtml(XMLStreamWriter out)
    throws XMLStreamException
  {
    out.writeStartElement("div");
    out.writeAttribute("id", "reference-legend");

    out.writeStartElement("form");
    out.writeAttribute("action", "#");

    out.writeStartElement("input");
    out.writeAttribute("type", "checkbox");
    out.writeAttribute("id", "reference-description-checkbox");
    out.writeAttribute("checked", "checked");
    out.writeAttribute("onchange", 
                       "toggleElementsByClass(this, 'reference-description')");
    //out.writeCharacters("description");
    out.writeEndElement();
    
    out.writeStartElement("label");
    out.writeAttribute("for", "reference-description-checkbox");
    out.writeCharacters("description");
    out.writeEndElement();

    out.writeStartElement("input");
    out.writeAttribute("type", "checkbox");
    out.writeAttribute("id", "reference-schema-checkbox");
    out.writeAttribute("checked", "checked");
    out.writeAttribute("onchange", 
                       "toggleElementsByClass(this, 'reference-schema')");
    //out.writeCharacters("schema");
    out.writeEndElement();
    
    out.writeStartElement("label");
    out.writeAttribute("for", "reference-schema-checkbox");
    out.writeCharacters("schema");
    out.writeEndElement();

    out.writeStartElement("input");
    out.writeAttribute("type", "checkbox");
    out.writeAttribute("id", "reference-attributes-checkbox");
    out.writeAttribute("checked", "checked");
    out.writeAttribute("onchange", 
                       "toggleElementsByClass(this, 'reference-attributes')");
    //out.writeCharacters("attributes");
    out.writeEndElement();

    out.writeStartElement("label");
    out.writeAttribute("for", "reference-attributes-checkbox");
    out.writeCharacters("attributes");
    out.writeEndElement();
    
    out.writeStartElement("input");
    out.writeAttribute("type", "checkbox");
    out.writeAttribute("id", "reference-example-checkbox");
    out.writeAttribute("checked", "checked");
    out.writeAttribute("onchange", 
                       "toggleElementsByClass(this, 'reference-example')");
    //out.writeCharacters("example");
    out.writeEndElement();

    out.writeStartElement("label");
    out.writeAttribute("for", "reference-example-checkbox");
    out.writeCharacters("examples");
    out.writeEndElement();

    out.writeEndElement();
    out.writeEndElement();
  }

  public void writeLaTeXTop(PrintWriter out)
    throws IOException
  {
    // irrelevant to latex
  }

  public void writeLaTeX(PrintWriter out)
    throws IOException
  {
    // irrelevant to latex
  }

  public void writeLaTeXEnclosed(PrintWriter out)
    throws IOException
  {
    // irrelevant to latex
  }

  public void writeLaTeXVerbatim(PrintWriter out)
    throws IOException
  {
    // irrelevant to latex
  }

  @Override
  public void writeAsciiDoc(PrintWriter out)
    throws IOException
  {
  }
}
